#include "GraphicsView.h"

#include <QWheelEvent>

#define DEFAULT_ZOOM_INCREMENT 0.1
#define DEFAULT_ZOOM_MAX       10.0
#define DEFAULT_ZOOM_MIN       0.2

/**
 * @brief GraphicsView::GraphicsView
 * @param parent
 */
GraphicsView::GraphicsView(QWidget * parent)
    : QGraphicsView(parent),
      m_zoomFactor(1)
{

}

/**

 */
GraphicsView::~GraphicsView()
{

}

/**
 * @brief GraphicsView::wheelEvent
 * @param event
 */
void GraphicsView::wheelEvent(QWheelEvent *event)
{
    Qt::KeyboardModifiers mod = event->modifiers();
    if(mod & Qt::ControlModifier)
    {
        int delta = event->delta();
        qreal factor = m_zoomFactor;
        if(delta > 0)
        {
            factor += (::abs(delta) / 120) * DEFAULT_ZOOM_INCREMENT;
        }
        else
        {
            factor -= (::abs(delta) / 120) * DEFAULT_ZOOM_INCREMENT;
        }

        setZoomFactor(factor);
        event->accept();
        return;
    }

    QGraphicsView::wheelEvent(event);
}

/**
    \brief  Sets the view zoom factor

    If the given factor is greater than the maximum zoom factor or lower than
    the minimum zoom factor, it will be capped to those values.

    \param  factor  Zoom factor
*/
void GraphicsView::setZoomFactor(qreal factor)
{
    if(factor > DEFAULT_ZOOM_MAX)
    {
        factor = DEFAULT_ZOOM_MAX;
    }

    if(factor < DEFAULT_ZOOM_MIN)
    {
        factor = DEFAULT_ZOOM_MIN;
    }

    m_zoomFactor = factor;

    setTransform(QTransform(QMatrix()));
    scale(m_zoomFactor, m_zoomFactor);
}
